#!/usr/bin/env python

from os import path
import sys
import yaml
from util import merge
from device import DeviceManager
from device_os import OS
from test import TestManager
from bds import BuildManager


################################
# Setup
################################
class Config:
    def __init__(self, file_name, private_file_name):
        self.data = self.load_config(file_name, private_file_name)

        self.build_manager = BuildManager(self.data.get('bds', None), OS.all)
        self.device_manager = DeviceManager(self.data.get('devices', None), OS.all)
        self.test_manager = TestManager(self.data.get('tests', None))

    def load_yaml(self, file_name, check):
        if not path.exists(file_name):
            if check:
                print "Current directory does not contain configuration file '%s'. Please create one and run again." % file_name
                sys.exit(1)
            else:
                return None

        with open(file_name, 'r') as stream:
            try:
                return yaml.load(stream)
            except yaml.YAMLError as exc:
                print(exc)

    def load_config(self, file_name, private_file_name):
        data = self.load_yaml(file_name, check=True)
        private_data = self.load_yaml(private_file_name, check=False)

        if private_data:
            private_data = merge(private_data, data)
            return private_data
        else:
            return data



#!/usr/bin/env python

import sys
from os import path, makedirs
import subprocess
import json
from device_os import OS
from util import Color, merge, TMP_DIR


################################
# Build Manager
#
# cli command: curl -O $(curl -u 77df53ebc3234f21862c4be0182dfd0a:'' -s http://inloop-bds.test.inloop.eu/api/v1/customers/medrio/projects/mcapture/applications/ios/environments/internal-calabash/builds/ | python -c 'import sys, json; print json.load(sys.stdin)["builds"][0]["download_url"]')
#
################################
class BuildManager(object):
    API_ENDPOINT = 'http://inloop-bds.test.inloop.eu/api/v1'

    def __init__(self, conf, os_list):
        if not conf:
            print Color.LIGHT_RED + 'BDS configuration is missing' + Color.ENDC
            sys.exit(1)

        conf['build_dir'] = conf.get('build_dir', path.join(TMP_DIR + 'builds/'))
        self.token = self.load(conf, 'token')
        self.customer = self.load(conf, 'customer')
        self.project = self.load(conf, 'project')

        self.builds = {}
        for os in os_list:
            self.load_builds(conf, os=os)

    # public
    def download(self, build):
        # load build from BDS
        bds_build = self.get_latest_bds_build(build)
        download_url = bds_build['download_url']
        print Color.BLUE + 'Downloading from url %s...' % download_url + Color.ENDC
        # download
        download_to = build.get_path()
        r = subprocess.call(['curl', '-o', download_to, download_url])
        if r == 0:
            print Color.BLUE + 'Downloaded to ' + download_to + Color.ENDC
            if build.os == OS.Android:
                print Color.BLUE + 'Resigning apk...' + Color.ENDC
                subprocess.call(['calabash-android', 'resign', download_to])
        else:
            print Color.LIGHT_RED + 'BDS build could not be downloaded' + Color.ENDC
            sys.exit(1)

    # public
    def check_and_get_build(self, os, build_name):
        build = self.get_build(os, build_name)

        build_path = build.get_path()
        if not path.exists(build_path):
            self.download(build)

        return build

    # public
    def get_build(self, os, build_name):
        if build_name:
            build = self.builds[os].get(build_name, None)
        else:
            build = self.get_default_build(os)

        if build:
            return build
        else:
            print Color.LIGHT_RED + "Build with name '%s' does not exists for '%s'!" % (build_name, os) + Color.ENDC
            sys.exit(1)

    # private
    def get_default_build(self, os):
        for name, build in self.builds[os].iteritems():
            if build.is_default:
                return build

        print Color.LIGHT_RED + "'%s' has no default build! Please add 'default: true' to one of the builds." % os + Color.ENDC
        sys.exit(1)

    # private
    def load(self, conf, prop):
        p = conf.get(prop, None)
        if not p:
            print Color.LIGHT_RED + "BDS configuration is missing a '%s'" % prop + Color.ENDC
            sys.exit(1)
        return p

    # private
    def load_builds(self, conf, os):
        builds = {}

        os_conf = conf.get(os, {})

        default_build_name = os_conf.get('default', None)
        default_build_found = False
        if not default_build_name:
            print Color.LIGHT_RED + "BDS is missing default build for '%s'" % os + Color.ENDC
            sys.exit(1)

        for name, info in os_conf.iteritems():
            if name == 'default':
                continue

            info['build_dir'] = info.get('build_dir', conf['build_dir'])
            build = Build(os, name, info)
            if name == default_build_name:
                build.is_default = True
                default_build_found = True
            builds[name] = build

        if not default_build_found:
            print Color.LIGHT_RED + "'%s' default build '%s' was not found" % (os, default_build_name) + Color.ENDC
            sys.exit(1)

        self.builds[os] = builds

    # private
    def get_latest_bds_build(self, build):
        token = "%s:\'\'" % self.token

        url = '{api}/customers/{customer}/projects/{project}/applications/{os}/'.format(
                api=self.API_ENDPOINT, customer=self.customer, project=self.project, os=build.os
        )
        if build.env:
            url += 'environments/{env}/'.format(env=build.env)
        if build.conf:
            url += 'configurations/{conf}/'.format(conf=build.conf)
        url += 'builds/'

        cmd = ['curl', '-u', token, '-s', url]
        c = ' '.join(cmd)

        proc = subprocess.Popen(c, shell=True, stdout=subprocess.PIPE)
        proc.wait()
        response = proc.communicate()[0]
        try:
            return json.loads(response)['builds'][0]
        except:
            print Color.LIGHT_RED + 'No BDS build was found' + Color.ENDC
            sys.exit(1)


class Build(object):
    def __init__(self, os, name, info):
        self.os = os
        self.name = name
        self.is_default = False
        self.app_id = info.get('app_id', None)
        if not self.app_id:
            print Color.LIGHT_RED + "BDS Build '%s' must specify an 'app_id'" % self.name + Color.ENDC
            sys.exit(1)
        self.env = info.get('env', None)
        self.conf = info.get('conf', None)
        self.build_dir = info['build_dir']
        # prepare path
        extension = '.apk' if os == OS.Android else '.ipa'
        self.file_name = name + extension

    # public
    def get_path(self):
        build_path = path.join(self.build_dir, self.os)
        if not path.exists(build_path):
            makedirs(build_path)
        return path.join(build_path, self.file_name)

    # public
    def show(self, line_start=''):
        if self.is_default:
            s = line_start + Color.LIGHT_RED + self.name + Color.RED + ' (default)'
        else:
            s = line_start + Color.LIGHT_GREEN + self.name

        s += '\n' + line_start + Color.YELLOW + '  - app ID: ' + Color.ENDC + self.app_id
        if self.env:
            s += '\n' + line_start + Color.YELLOW + '  - env: ' + Color.ENDC + self.env
        if self.conf:
            s += '\n' + line_start + Color.YELLOW + '  - conf: ' + Color.ENDC + self.conf
        return s



#!/usr/bin/env python

from os import environ, makedirs, path
import time
import sys
import subprocess
import shutil
from util import Color, merge, TMP_DIR
from device_os import OS


################################
# Device Manager
################################
class DeviceManager(object):
    def __init__(self, conf, os_list):
        if not conf:
            print Color.LIGHT_RED + 'DEVICES configuration is missing' + Color.ENDC
            sys.exit(1)

        self.devices = {}
        for os in os_list:
            self.load_devices(conf, os=os)

    # private
    def load_devices(self, conf, os):
        for name, info in conf.get(os, {}).iteritems():
            if os == OS.Android:
                self.devices[name] = AndroidDevice(name)
            elif os == OS.iOS:
                self.validate_device(name, info, 'uuid')
                self.validate_device(name, info, 'ip')

                self.devices[name] = IosDevice(name, info['uuid'], info['ip'])

    # private
    def validate_device(self, name, params, param_name):
        if param_name not in params.keys():
            print Color.LIGHT_RED + "Device '%s' is missing parameter '%s'" % (name, param_name) + Color.ENDC
            sys.exit(1)

    # public
    def get_device(self, name):
        device = self.devices.get(name, None)
        if device:
            return device
        else:
            print Color.LIGHT_RED + "Device '%s' was not found" % name + Color.ENDC
            sys.exit(1)


################################
# Base Device
################################
class BaseDevice(object):
    def __init__(self, os, name):
        self.os = os
        self.name = name
        self.ENV = environ.copy()

    def call(self, cmd):
        subprocess.call(cmd, env=self.ENV)

    def run_console(self, build):
        cmd = self.get_console_cmd(build)
        self.call(cmd)

    def get_console_cmd(self, build):
        return []  # implement

    def run(self, build, test, report=False):
        cmd = self.get_run_cmd(build)
        self.show_and_run(cmd, test, report)

    def get_run_cmd(self, build):
        return []  # implement

    def install(self, build):
        pass  # implement

    def uninstall(self, build):
        pass  # implement

    def show_and_run(self, base_cmd, test, report):
        tmp = TMP_DIR
        tmp_out = self.current_report_dir(tmp)
        dir_out = self.device_reports_dir(test.output_dir)

        cmd = base_cmd + test.create_command(tmp_out, report)
        self.ENV = merge(test.env, self.ENV)
        # show commands
        print '--------------------------------------------------------------------------'
        print '| Commands: '
        print '|'
        print '|', " ".join(cmd)
        print '|'
        print '| NOTE: output files will be moved to:', dir_out
        print '|'
        print '--------------------------------------------------------------------------'

        # run command
        self.ENV["SCREENSHOT_PATH"] = tmp_out + '/'  # has to end with '/'
        self.call(cmd)

        # move files if necessary
        if tmp != test.output_dir:
            shutil.move(tmp_out, dir_out)
            print "----> remove: ", self.reports_dir(tmp)
            shutil.rmtree(self.reports_dir(tmp))

    def show(self, line_start=''):
        return line_start + Color.LIGHT_GREEN + '%s ' % self.name + Color.YELLOW + '(%s)' % self.os + Color.ENDC

    def reports_dir(self, parent=None):
        dir = 'reports/'
        if parent:
            dir = path.join(parent, dir)
            if not path.exists(dir):
                makedirs(dir)
        return path.abspath(dir)

    def device_reports_dir(self, parent=None):
        dir = path.join(self.reports_dir(parent), '%s-%s/' % (self.os, self.name))
        if not path.exists(dir):
            makedirs(dir)
        return path.abspath(dir)

    def current_report_dir(self, parent=None):
        dir = path.join(self.device_reports_dir(parent), time.strftime('%Y_%m_%d-%H_%M_%S'))
        if not path.exists(dir):
            makedirs(dir)
        return dir


################################
# iOS Device
################################
class IosDevice(BaseDevice):
    CLI_TOOL = 'ideviceinstaller'

    def __init__(self, name, uuid, ip):
        super(IosDevice, self).__init__(OS.iOS, name)
        self.ENV["DEVICE_TARGET"] = uuid
        self.ENV["DEVICE_ENDPOINT"] = 'http://%s:37265' % ip

    def get_console_cmd(self, build):
        self.ENV["BUNDLE_ID"] = build.app_id
        return ['calabash-ios', 'console', '-p', 'ios']

    def get_run_cmd(self, build):
        self.ENV["BUNDLE_ID"] = build.app_id
        return ['cucumber', '-p', 'ios']

    def show(self, line_start=''):
        s = super(IosDevice, self).show(line_start=line_start)
        s += '\n' + line_start + Color.YELLOW + '  - UUID: ' + Color.ENDC + '%s' % self.ENV[
            "DEVICE_TARGET"] + Color.ENDC
        s += '\n' + line_start + Color.YELLOW + '  - IP: ' + Color.ENDC + '%s' % self.ENV[
            "DEVICE_ENDPOINT"] + Color.ENDC
        return s

    def check_cli_tool(self):
        self.call(['brew', 'install', self.CLI_TOOL])

    def install(self, build):
        self.check_cli_tool()
        self.call([self.CLI_TOOL, '-i', build.get_path()])

    def uninstall(self, build):
        self.check_cli_tool()
        self.call([self.CLI_TOOL, '-U', build.app_id])


################################
# Android Device
################################
class AndroidDevice(BaseDevice):
    CLI_TOOL = 'adb'

    def __init__(self, name):
        super(AndroidDevice, self).__init__(OS.Android, name)

    def get_console_cmd(self, build):
        return ['calabash-android', 'console', build.get_path(), '-p', 'android']

    def get_run_cmd(self, build):
        return ['calabash-android', 'run', build.get_path(), '-p', 'android']

    def check_cli_tool(self):
        self.call(['brew', 'install', self.CLI_TOOL])

    def install(self, build):
        self.call(['calabash-android', 'build', build.get_path()])  # rebuild test-server
        self.call([self.CLI_TOOL, 'install', '-r', build.get_path()])  # install app

    def uninstall(self, build):
        self.call([self.CLI_TOOL, 'uninstall', build.app_id])



#!/usr/bin/env python

TMP_DIR = '.capy/'

class Color:
    ENDC = '\033[0m'

    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    DEFAULT = '\033[39m'
    BLACK = '\033[30m'
    RED = '\033[31m'
    GREEN = '\033[32m'
    YELLOW = '\033[33m'
    BLUE = '\033[34m'
    MANGETA = '\033[35m'
    CYAN = '\033[36m'
    LIGHT_GRAY = '\033[37m'
    DARK_GRAY = '\033[90m'
    LIGHT_RED = '\033[91m'
    LIGHT_GREEN = '\033[92m'
    LIGHT_YELLOW = '\033[93m'
    LIGHT_BLUE = '\033[94m'
    LIGHT_MANGETA = '\033[95m'
    LIGHT_CYAN = '\033[96m'
    WHITE = '\033[97m'


def merge(user, default):
    if isinstance(user, dict) and isinstance(default, dict):
        for k, v in default.iteritems():
            if k not in user:
                user[k] = v
            else:
                user[k] = merge(user[k], v)
    return user

def get(conf, prop, default):
    p = conf.get(prop, None)
    if p:
        return p
    else:
        return default